#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
from collections import defaultdict
import json

KneserNey = False  # Soll Kneser-Ney-Glättung verwendet werden?

if len(sys.argv) != 4:
    sys.exit('Usage: '+sys.argv[0]+' n-gram-length text-file model-file')

    #Kommandozeilen-Argumente
L = int(sys.argv[1])       # Länge der verwendeten N-Gramme
trainFile = sys.argv[2] # Datei mit den Trainingsdaten
paramFile = sys.argv[3] # Ausgabedatei

# Trainingstext einlesen
with open(trainFile) as file:
    text = file.read()

# N-Gramm-Häufigkeiten berechnen
ngramfreq = defaultdict(int)
for i in range(len(text) - L+1):
    ngram = text[i:i+L]
    ngramfreq[ngram] += 1

# Wahrscheinlichkeiten für alle NGramm-Längen berechnen
prob = {}
for _ in range(L):

    # Berechnung des Discounts
    N1 = sum(1 for v in ngramfreq.values() if v == 1)
    N2 = sum(1 for v in ngramfreq.values() if v == 2)

    # Discount berechnen
    # Der Discount ist 0.5, falls nur wenige N-Gramme genau einmal auftraten.
    discount = N1 / (N1 + 2.0 * N2)  if N1 > 100  else  0.5

    # Berechnung aller Kontexthäufigkeiten
    contextfreq = defaultdict(int)
    for ngram, freq in ngramfreq.items():
        context = ngram[:-1]
        contextfreq[context] += freq

    # Berechnung der relativen Häufigkeiten mit Discount
    for ngram, f in ngramfreq.items():
        context = ngram[:-1]
        prob[ngram] = (ngramfreq[ngram] - discount) / contextfreq[context]

    # Berechnung der N-1Gramm-Häufigkeiten
    sngramfreq = defaultdict(int)
    for ngram, freq in ngramfreq.items():
        sngramfreq[ngram[1:]] += 1  if KneserNey  else  freq
    # N-Gramm-Tabelle durch N-1-Grammtabelle ersetzen und iterieren
    ngramfreq = sngramfreq

# Parameter in Datei speichern
with open(paramFile,"w") as file:
    json.dump(prob, file)
